import { Node, MemberExpression } from '@babel/types';
import { MagicStringAST } from 'magic-string-ast';

interface BaseNode {
    type: string;
}
interface Code extends BaseNode {
    type: 'Code';
    content: string;
    priority: number;
}
interface HoistedCode extends BaseNode {
    type: 'HoistedCode';
    content: string;
    priority: number;
}
interface Property extends BaseNode {
    type: 'Property';
    name: string;
    source: unknown;
    exposed: boolean;
}
type TransformNode = Code | HoistedCode | Property;
type VisitNode = Code | HoistedCode;
declare const factory: {
    priority: {
        readonly gap: 1;
        readonly dependency: -2;
        readonly interface: -1;
        readonly derived: 1;
        readonly effect: 2;
    };
    code: (content: string, priority?: number) => Code;
    hoist: (content: string, priority?: number) => HoistedCode;
    property: (name: string, source: unknown, exposed?: boolean) => Property;
};
interface TransformOptions {
    /**
     * Whether to generate <script setup> block.
     * @default true
     */
    scriptSetup: boolean;
    /**
     * Whether to generate code using [Reactivity Transform]{@link https://vue-macros.dev/features/reactivity-transform.html}.
     * @default false
     */
    reactivityTransform: boolean;
    /**
     * Whether to destructure props defined by `defineProps`.
     * Only valid when `scriptSetup` is `true`.
     * @default true
     */
    propsDestructure: boolean;
    /**
     * Specifies a mapping of import names.
     * For example `{ 'vue-router': 'vue-router-v4' }`.
     * @default {}
     */
    aliases: Record<string, string>;
    /**
     * Specifies additional plugins.
     * Plugins that transform Vue's built-in options are always enabled.
     * @default [...builtinPlugins]
     */
    plugins: Plugin[];
}
interface TransformContext {
    name: string;
    node: Node;
    magicString: MagicStringAST;
    options: TransformOptions;
}
interface VisitContext {
    name: string;
    node: MemberExpression;
    magicString: MagicStringAST;
    source: unknown;
}
interface TransformHelpers {
    factory: typeof factory;
    transform: (node: Node) => ReturnType<NonNullable<Plugin['transform']>>;
}
interface VisitHelpers {
    factory: typeof factory;
}
interface Plugin {
    transformInclude?: (context: TransformContext) => boolean;
    transform?: (context: TransformContext, helpers: TransformHelpers) => Generator<TransformNode, void | false, unknown>;
    visitProperty?: (context: VisitContext, helpers: VisitHelpers) => Generator<VisitNode, void | string, unknown>;
}
declare function defineSpinachPlugin(plugin: Plugin): Plugin;

declare function transformSFC(code: string, userOptions?: Partial<TransformOptions>): string;

export { type Plugin, type TransformOptions, defineSpinachPlugin, transformSFC };
